﻿--[[
	MrtWoo_AntiSpam: Detects and blocks russian commercial spam and flud
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local MrtWoo = MrtWoo;
local AntiSpam = MrtWoo:NewModule("AntiSpam", "AceEvent-3.0");
AntiSpam.Version = "r106";

local L = LibStub("AceLocale-3.0"):GetLocale("MrtWoo_AntiSpam", true);

local opts = {
	af_options = {
		type="group",
		name=L["Anti-Flud"],
		args={
			antiflud = {
				type = "toggle",
				name = L["Anti-Flud"],
				desc = L["Enable Anti-flud filters"],
				get = function() return MrtWoo.db.profile.modules.as_antiflud end,
				set = function() MrtWoo.db.profile.modules.as_antiflud = not MrtWoo.db.profile.modules.as_antiflud end,
				order=1,
			},			

			antiflud_time = {
				type = "range",
				max = 1200,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["How often is allowed to send the same message?"],
				desc = L["The larger this value, the less will appear the same message"],
				get = function() return MrtWoo.db.profile.modules.as_antiflud_time end,
				set = function(info, input) MrtWoo.db.profile.modules.as_antiflud_time = input end,
				order=2,
			},
			
			f_check = {
				type = "description",
				name = L["What types of messages to check?"],
				order=3,			
			},
						
			f_check_say = {
				type = "toggle",
				name = L["Say"],
				desc = L["Check the say?"],
				get = function() return MrtWoo.db.profile.modules.as_f_check_say end,
				set = function() MrtWoo.db.profile.modules.as_f_check_say = not MrtWoo.db.profile.modules.as_f_check_say end,
				order=4,
			},
			
			f_check_yell = {
				type = "toggle",
				name = L["Yell"],
				desc = L["Check the yell?"],
				get = function() return MrtWoo.db.profile.modules.as_f_check_yell end,
				set = function() MrtWoo.db.profile.modules.as_f_check_yell = not MrtWoo.db.profile.modules.as_f_check_yell end,
				order=5,
			},
			--[[
			f_check_whisper = {
				type = "toggle",
				name = L["Whisper"],
				desc = L["Check the whisper?"],
				get = function() return MrtWoo.db.profile.modules.as_f_check_whisper end,
				set = function() MrtWoo.db.profile.modules.as_f_check_whisper = not MrtWoo.db.profile.modules.as_f_check_whisper end,
				order=6,
			},
			]]
			f_check_channel = {
				type = "toggle",
				name = L["Channels"],
				desc = L["Check the channels?"],
				get = function() return MrtWoo.db.profile.modules.as_f_check_channel end,
				set = function() MrtWoo.db.profile.modules.as_f_check_channel = not MrtWoo.db.profile.modules.as_f_check_channel end,
				order=7,
			},
		}			
	},
	as_options = {
		type="group",
		name=L["AntiSpam"],
		args={
			silent = {
				type = "toggle",
				name = L["Quiet mode"] ,
				desc = L["Do not display messages when blocking spam"],
				get = function() return MrtWoo.db.profile.modules.as_silent end,
				set = function() MrtWoo.db.profile.modules.as_silent = not MrtWoo.db.profile.modules.as_silent end,
				order=3,
			},
			
			report = {
				type = "toggle",
				name = L["Send reports of spam"],
				desc = L["If enabled, the addon will automatically send reports of spam"],
				get = function() return MrtWoo.db.profile.modules.as_report end,
				set = function() MrtWoo.db.profile.modules.as_report = not MrtWoo.db.profile.modules.as_report end,
				order=3,
			},
			
			s_check = {
				type = "description",
				name = L["What types of messages to check?"],
				order=4,			
			},
						
			s_check_say = {
				type = "toggle",
				name = L["Say"],
				desc = L["Check the say?"],
				get = function() return MrtWoo.db.profile.modules.as_s_check_say end,
				set = function() MrtWoo.db.profile.modules.as_s_check_say = not MrtWoo.db.profile.modules.as_s_check_say end,
				order=5,
			},
			
			s_check_yell = {
				type = "toggle",
				name = L["Yell"],
				desc = L["Check the yell?"],
				get = function() return MrtWoo.db.profile.modules.as_s_check_yell end,
				set = function() MrtWoo.db.profile.modules.as_s_check_yell = not MrtWoo.db.profile.modules.as_s_check_yell end,
				order=6,
			},
			--[[
			s_check_whisper = {
				type = "toggle",
				name = L["Whisper"],
				desc = L["Check the whisper?"],
				get = function() return MrtWoo.db.profile.modules.as_s_check_whisper end,
				set = function() MrtWoo.db.profile.modules.as_s_check_whisper = not MrtWoo.db.profile.modules.as_s_check_whisper end,
				order=7,
			},
			]]
			s_check_channel = {
				type = "toggle",
				name = L["Channels"],
				desc = L["Check the channels?"],
				get = function() return MrtWoo.db.profile.modules.as_s_check_channel end,
				set = function() MrtWoo.db.profile.modules.as_s_check_channel = not MrtWoo.db.profile.modules.as_s_check_channel end,
				order=8,
			},			
			
			patterns_editor = {
				type = "execute",
				name = L["Patterns Editor"],
				desc = L["Show Patterns Editor"],
				func = function(info, input) AntiSpam:ShowPatternsEditor(nil); end,
				order=9,
			},
						
			spam_rate = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Maximum rating of the message after which it will be blocked"],
				desc = L["After receiving the message is scanned for spam and it is assigned a rating, usually a normal message is not spam has a rating of 0-75 ... If the message rating exceeds a given threshold of the lock, then the message will be seen as spam and all messages from this user will be freezing."],
				get = function() return MrtWoo.db.profile.modules.as_spamrate end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spamrate = input end,
				order=10,
			},
			
			spam_additional_checks = {
				type = "toggle",
				name = L["Perform additional checks?"],
				desc = L["It often happens that the scan only messages are not enough ... so sometimes you need to check the level of character, his nickname ..."],
				get = function() return MrtWoo.db.profile.modules.as_spam_additional_checks end,
				set = function() MrtWoo.db.profile.modules.as_spam_additional_checks = not MrtWoo.db.profile.modules.as_spam_additional_checks end,
				order=11,
			},
						
			spam_rate_needed_for_additional_checks = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Rating posts required for the additional checks"],
				desc = L["This value must be less than the rating of the lock, or additional tests will not meet"],
				get = function() return MrtWoo.db.profile.modules.as_spamrate_needed_for_additional_checks end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spamrate_needed_for_additional_checks = input end,
				order=12,
			},
						
			spam_maxlvl = {
				type = "range",
				max = 85,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Max is not a reliable player level"],
				desc = L["Spammers usually do not get the level higher than 15 ..."],
				get = function() return MrtWoo.db.profile.modules.as_spam_maxlvl end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spam_maxlvl = input end,
				order=13,
			},

			spam_ratelvl = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Extra rating posts if the author is not a reliable level"],
				desc = L["Specify additional rating messages, it will be added to the rating message that was received after checking the contents ... if the total rating reports exceed the threshold of the lock, then the message will be seen as spam and all messages from this user will be freezing"],
				get = function() return MrtWoo.db.profile.modules.as_spam_ratelvl end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spam_ratelvl = input end,
				order=14,
			},		

			gc_wait_time = {
				type = "range",
				max = 600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = L["Garbage collection interval (sec)"],
				desc = L["This value affects how often will occur trying to remove unnecessary data from memory"],
				get = function() return MrtWoo.db.profile.modules.as_gc_wait_time end,
				set = function(info, input) MrtWoo.db.profile.modules.as_gc_wait_time = input end,
				order=15,
			},
			
			debug = {
				type = "toggle",
				name = L["Debug"],
				desc = L["It makes sense to enable in the search errors (displays additional information during the addon)"],
				get = function() return MrtWoo.db.profile.modules.as_debug end,
				set = function() MrtWoo.db.profile.modules.as_debug = not MrtWoo.db.profile.modules.as_debug end,
				order=16,
			},
		}
	}
};

function AntiSpam:SetDefaults()
	MrtWoo.defaults.profile.modules.as_silent = true;
	MrtWoo.defaults.profile.modules.as_debug = false;
	MrtWoo.defaults.profile.modules.as_antiflud = true;
	MrtWoo.defaults.profile.modules.as_report = false;
	MrtWoo.defaults.profile.modules.as_antiflud_time = 300;
	MrtWoo.defaults.profile.modules.as_spam_additional_checks = true;
	MrtWoo.defaults.profile.modules.as_spam_maxlvl = 15;
	MrtWoo.defaults.profile.modules.as_spam_ratelvl = 50;
	MrtWoo.defaults.profile.modules.as_spamrate = 100;
	MrtWoo.defaults.profile.modules.as_spamrate_needed_for_additional_checks = 50;
	MrtWoo.defaults.profile.modules.as_gc_wait_time = 500;
	MrtWoo.defaults.profile.modules.as_gc_message_lifetime = 60;
	MrtWoo.defaults.profile.modules.as_gc_checkinfo_lifetime = 600;
	MrtWoo.defaults.profile.modules.as_filters = false;
	MrtWoo.defaults.profile.modules.as_blocklist = false;
	
	MrtWoo.defaults.profile.modules.as_f_check_say = true;
	MrtWoo.defaults.profile.modules.as_f_check_yell = true;
	MrtWoo.defaults.profile.modules.as_f_check_whisper = true;
	MrtWoo.defaults.profile.modules.as_f_check_channel = true;
	
	MrtWoo.defaults.profile.modules.as_s_check_say = true;
	MrtWoo.defaults.profile.modules.as_s_check_yell = true;
	MrtWoo.defaults.profile.modules.as_s_check_whisper = true;
	MrtWoo.defaults.profile.modules.as_s_check_channel = true;	
end;

function AntiSpam:OnInitialize()
	self:SetDefaults();
	table.insert(MrtWoo.options.plugins, opts);
	
	MrtWoo:RegisterChatCommand("mrtwoo_show_smi", AntiSpam.ShowSpamMonInfo);
	MrtWoo:RegisterChatCommand("mrtwoo_patterns", AntiSpam.ShowPatternsEditor);
	MrtWoo:RegisterChatCommand("mrtwoo_test", AntiSpam.AsTest);
end;

function AntiSpam:OnClosePatternsEditor()
	-- MrtWoo:Print("=== AntiSpam:OnClosePatternsEditor() ===");
	MrtWoo.db.profile.modules.as_filters = MrtWoo_AntiSpamFilterGUI.Words;
	MrtWoo_AntiSpamFilter.Words = MrtWoo_AntiSpamFilterGUI.Words;
	AntiSpam.Monitor.MessagesFilter:ReloadWords();
end;

function AntiSpam:ShowSpamMonInfo(input)
	-- AntiSpam.MsgsQueue:Show();
	AntiSpam.Monitor:ShowSMI();
end;

function AntiSpam:ShowPatternsEditor(input)
	local FilterGUI = MrtWoo_AntiSpamFilterGUI:new();
	FilterGUI:CreateFrame();
	FilterGUI:Show();
end;

function AntiSpam:AsTest(input)
	AntiSpam.Monitor:ShowSpamers();
end;

function AntiSpam:UnitPopup_ShowMenu(dropdownMenu, which, unit, name, userData, ...)
	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = _G["DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i];
		if (button.value and string.find(button.value , "MRTWOO_BLOCK_") ~= nil) then
			button.func = UnitPopupButtons[button.value].func;
		end;
	end;
end;

function AntiSpam_OnClickBlockButton(self)
	local dropdownFrame = UIDROPDOWNMENU_INIT_MENU
	local name = dropdownFrame.name
	local button = self.value;
	if ( name ) then 
		local dialog = StaticPopup_Show("CONFIRM_MRTWOO_BLOCK", name);
		if ( dialog ) then
			dialog.data = {button = button, name = name};
		end;
	end
end;

function AntiSpam:OnEnable()
	self.Monitor = MrtWoo_AntiSpamMonitor:new();
	
	self:RegisterMessage('MRTWOO_ASMON_PROCESSED_MESSAGE', AntiSpam.SendProcessedMessage);
	self:RegisterMessage('MRTWOO_ASGUI_ONCLOSE', AntiSpam.OnClosePatternsEditor);
	
	if MrtWoo.db.profile.modules.as_filters ~= false then
		MrtWoo_AntiSpamFilter.Words = MrtWoo.db.profile.modules.as_filters;
		self.Monitor.MessagesFilter:ReloadWords();
	end;
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", AntiSpam.MessagesEventFilter);
	--[[
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_DND", AntiSpam.MessagesEventFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_AFK", AntiSpam.MessagesEventFilter);
	]]
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
	
	UnitPopupButtons["MRTWOO_BLOCK"] = {};
	UnitPopupButtons["MRTWOO_BLOCK"].dist = 0;
	UnitPopupButtons["MRTWOO_BLOCK"].nested = 1;
	UnitPopupButtons["MRTWOO_BLOCK"].text = L["Block for..."];

	UnitPopupButtons["MRTWOO_BLOCK_15M"] = {dist = 0, text = string.format(L["%s min"], 15), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_30M"] = {dist = 0, text = string.format(L["%s min"], 30), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_60M"] = {dist = 0, text = string.format(L["%s min"], 60), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_120M"] = {dist = 0, text = string.format(L["%s min"], 120), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_24H"] = {dist = 0, text = string.format(L["%s hours"], 24), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_48H"] = {dist = 0, text = string.format(L["%s hours"], 48), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_7D"] = {dist = 0, text = string.format(L["%s days"], 7), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_15D"] = {dist = 0, text = string.format(L["%s days"], 15), func = AntiSpam_OnClickBlockButton};
	UnitPopupButtons["MRTWOO_BLOCK_30D"] = {dist = 0, text = string.format(L["%s days"], 30), func = AntiSpam_OnClickBlockButton};
	
	UnitPopupMenus["MRTWOO_BLOCK"] = { 
		"MRTWOO_BLOCK_15M",
		"MRTWOO_BLOCK_30M",
		"MRTWOO_BLOCK_60M",
		"MRTWOO_BLOCK_120M",
		"MRTWOO_BLOCK_24H",
		"MRTWOO_BLOCK_48H",
		"MRTWOO_BLOCK_7D",
		"MRTWOO_BLOCK_15D",
		"MRTWOO_BLOCK_30D",
	};

	StaticPopupDialogs["CONFIRM_MRTWOO_BLOCK"] = {
		text = "Вы действительно хотите заблокировать |3-1(%s)?",
		button1 = ACCEPT,
		button2 = CANCEL,
		OnAccept = function(self, data)
			local name = data.name;
			local button = data.button;
			
			local lifetime = 0;
			if (button == "MRTWOO_BLOCK_15M") then
				lifetime = 60 * 15;
			elseif (button == "MRTWOO_BLOCK_30M") then
				lifetime = 60 * 30;
			elseif (button == "MRTWOO_BLOCK_60M") then
				lifetime = 60 * 60;
			elseif (button == "MRTWOO_BLOCK_120M") then
				lifetime = 60 * 120;
			elseif (button == "MRTWOO_BLOCK_24H") then
				lifetime = 86400;
			elseif (button == "MRTWOO_BLOCK_48H") then
				lifetime = 86400 * 2;
			elseif (button == "MRTWOO_BLOCK_7D") then
				lifetime = 86400 * 7;
			elseif (button == "MRTWOO_BLOCK_15D") then
				lifetime = 86400 * 15;
			elseif (button == "MRTWOO_BLOCK_30D") then
				lifetime = 86400 * 30;
			end;
			
			if lifetime > 0 then
				MrtWoo_AntiSpamBlockList:Block(name, lifetime, "User blocked manually");
			end;
					
		end,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	};
	
	hooksecurefunc("UnitPopup_ShowMenu", AntiSpam.UnitPopup_ShowMenu);
	
	tinsert(UnitPopupMenus["FRIEND"], #UnitPopupMenus["FRIEND"]-1, "MRTWOO_BLOCK");	
	
	
end;



function AntiSpam:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SAY", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_YELL", AntiSpam.MessagesEventFilter);
	--[[	
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_EMOTE", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_DND", AntiSpam.MessagesEventFilter);
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_AFK", AntiSpam.MessagesEventFilter);
	]]
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
end;

function AntiSpam:SendProcessedMessage(msgType, ...)
	if msgType == nil then
		return false;
	end;
	
	local old_self = self;

	for i=1,NUM_CHAT_WINDOWS do			
		local chatWin = getglobal("ChatFrame"..i);
		if chatWin then			
			local channels = {GetChatWindowMessages(chatWin:GetID())}
			for x=1,#channels do
				if channels[x] == msgType then					

					self = chatWin;

					self.mrtwoo_spm = true;
					ChatFrame_OnEvent(self, ...);
					self.mrtwoo_spm = false;

					self = old_self;

					break;
				end;
			end;
		end;
	end;
	
	return true;
end;

function AntiSpam:MessagesEventFilter(event, ...)
	local msg  = select(1, ...);
	local author = select(2, ...);
	
	if self.mrtwoo_spm or 
		(string.len(msg) == 0) or 
		(string.len(author) == 0) or 
		(author == "*") -- Don't check lfgforwarder messagess
	then
		return false, ...;
	end;
	
	local msgId = select(11, ...);	
		
	AntiSpam.Monitor:AddMessageToQueue(msgId, event, ...);
		
	return true, ...;
end;
